"""
AIPTX Beast Mode - Specialized Credential Harvesters
====================================================

Platform-specific credential extraction modules.
"""

from __future__ import annotations

from aipt_v2.post_exploit.harvesters.file_search import (
    FileSearchHarvester,
    search_credential_files,
)
from aipt_v2.post_exploit.harvesters.cloud_metadata import (
    CloudMetadataHarvester,
    harvest_cloud_metadata,
)
from aipt_v2.post_exploit.harvesters.env_secrets import (
    EnvSecretHarvester,
    harvest_env_secrets,
)

__all__ = [
    "FileSearchHarvester",
    "search_credential_files",
    "CloudMetadataHarvester",
    "harvest_cloud_metadata",
    "EnvSecretHarvester",
    "harvest_env_secrets",
]
